/**
  \file   a2system.h
  \brief      - 
  \author  
  \date   13.12.1990
*/

#ifndef H_A2SYSTEM
#define H_A2SYSTEM
#pragma pack (push, 8)

#ifndef H_A2SYSDEF
# include "a2sysdef.h"
#endif

#ifndef H_A2STR_H
# include "../include/a2str.h"
#endif

#ifndef __FCNTL_H
# ifndef _INC_FCNTL
#  ifndef O_RDONLY
#   include <fcntl.h>
#   include <share.h>
#  endif
# endif
#endif

/*------------------------------------------------------------------*/
/*------------------------------------------------------------------*/

#if !defined(_MSC_VER) || (_MSC_VER < 1400)
# define strcpy_s(dst,sz, src)   strcpy(dst, src)
# define strncpy_s(dst,sz,src,n) strncpy(dst, src, n)
# define strcat_s(dst,sz, src)   strcat(dst, src)
# define strncat_s(dst,sz,src,n) strncat(dst, src, n)
# define s_stricmp               stricmp
# define s_strnicmp              strnicmp
# define s_strupr(s,n)           strupr(s)
# define s_strlwr(s,n)           strlwr(s)
# define s_strset(s,sz, c)       strset(s,c)
# define s_fopen                 fopen
# define s_sscanf                sscanf
#else
# define s_stricmp               _stricmp
# define s_strnicmp              _strnicmp
# define s_strupr(s,n)           _strupr_s(s,n)
# define s_strlwr(s,n)           _strlwr_s(s,n)
# define s_strset(s,sz, c)       _strset_s(s,sz,c)
  __inline FILE* s_fopen ( const char* fname, const char* mode )
  { FILE* pf=NULL; return fopen_s(&pf,fname,mode) ? NULL : pf; }
# define s_sscanf                sscanf_s
#endif

#ifdef _MSC_VER
# if _MSC_VER < 1400
#  define snprintf               _snprintf
#  define vsnprintf              _vsnprintf
# else
#  define snprintf               sprintf_s
#  define vsnprintf(bf,sz,fm,ar) _vsnprintf_s(bf,sz,sz,fm,ar)
#  define strdup                 _strdup
# endif
#endif

/*------------------------------------------------------------------*/
/*---------------------------- T Y P E S ---------------------------*/
/*------------------------------------------------------------------*/

#include <limits.h>
#include <stdlib.h>

#ifdef __FOR_WINDOWS__
 typedef void _huge * V_PTR;
#else
 typedef char * LPSTR;
 typedef const char * LPCSTR;
 typedef void * V_PTR;
#endif

#ifndef _IO_H
# ifndef __IO_H
#  include <io.h>
# endif
#endif

#ifndef __FOR_WIN32__
# ifndef __DOS_DEF_
#  ifndef __DOS_H
#   include <dos.h>
#  endif
# endif
#endif

#ifdef __TURBOC__

typedef struct ftime FTIME;

#ifndef __DIR_DEF_
# ifndef __DIR_H
#  include <dir.h>
# endif
#endif
typedef struct ffblk FFBLK;

#else /*not __TURBOC__*/

#include <direct.h>

#ifndef MAXPATH
# define  MAXPATH _MAX_PATH
# define  MAXDIR  _MAX_DIR
# define  MAXFILE _MAX_FNAME
# ifdef  _MAX_DRIVE
#  define MAXDRIVE _MAX_DRIVE
# else
#  define MAXDRIVE 3
# endif
# ifdef  _MAX_EXT
#  define MAXEXT   _MAX_EXT
# else
#  define MAXEXT   256
# endif
#endif

typedef struct {
   unsigned ft_sec   : 5;
   unsigned ft_min   : 6;
   unsigned ft_hour  : 5;
   unsigned ft_day   : 5;
   unsigned ft_month : 4;
   unsigned ft_year  : 7;
} FTIME;

typedef struct {
   char     ff_reserved[21];
   char     ff_attrib;
   unsigned ff_ftime;
   unsigned ff_fdate;
   long     ff_fsize;
#if !defined(__FOR_DPMI32__) && !defined(__FOR_WIN32__)
   char     ff_name[13];
#else
   char     ff_name[260];
#endif
#if defined(__FOR_WIN32__)
   long   handle;
   struct _finddata_t ft;
#endif
} FFBLK;

# if !defined(__FOR_DPMI32__) && !defined(__FOR_WIN32__)
#  ifndef MK_FP
#   define MK_FP(seg,ofs)  ((void *) \
            (((unsigned long)(seg) << 16) | (unsigned)(ofs)))
#  endif
# endif

#endif /*__TURBOC__*/

#ifndef FA_RDONLY
# define FA_RDONLY  0x01
# define FA_HIDDEN  0x02
# define FA_SYSTEM  0x04
# define FA_LABEL   0x08
# define FA_DIREC   0x10
# define FA_ARCH    0x20
#endif

/*------------------------------------------------------------------*/
/*---------------------- F U N C T I O N S -------------------------*/
/*------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

A2EXT LPCSTR A2PAS a2ROOTWORD(void);

void a2wake_up();
void a2clean_up();

int os_err (int errc);

#if defined(__FOR_WINDOWS__) || defined(__FOR_WIN32__) || defined(__FOR_DPMI32__)
# define OS_SETNFILES(NFILES) SetHandleCount(NFILES)
#else
# if defined(__DPMI16__)
#  define OS_SETNFILES(NFILES) {_BX=NFILES; _AH=0x67; geninterrupt(0x21);}
# else
#  define OS_SETNFILES(NFILES)
# endif
#endif

#define osopen(fname) os_open(fname,O_RDWR)
int os_open (LPCSTR fname, int oflags);

#define osclose os_close
int os_close (int fh);

#define osfexist os_exist
int os_exist (LPSTR fname);

#define oscreat(fname) os_creat(fname,0)
int os_creat (LPCSTR fname, int attrib);

int os_read (int fh, V_PTR buf, unsigned len);

int os_write(int fh, V_PTR buf, unsigned len);

int osread (int fn, V_PTR buf, long offset, int blksz);
int oslsrd (int fn, V_PTR buf, long blkn,   int blksz);
int oswrite(int fn, V_PTR buf, long offset, int blksz);
int oslswrt(int fn, V_PTR buf, long blknmb, int blksz);

int os_flush (int fh);

long os_lseek (int fh, long offset, int origin);
int os_lseek1 (int fh, long blk, int blksz);

#define osexepath os_exepath
LPSTR os_exepath(LPSTR exedir);

#define osopeng(fname) os_openg(fname,O_RDWR)
int os_openg (LPSTR fname, int oflags);
int os_opgmode(int mode);
/* global search: 0 - curdir, exedir, nikadir, path */
/*                1 - exedir, curdir, nikadir, path */

A2EXT int A2PAS os_fullnm (LPSTR fname, LPSTR name, char disp);
void os_fnsplit(LPSTR path, LPSTR drive, LPSTR dir, LPSTR name, LPSTR ext);
void os_fnmerge(LPSTR path, LPSTR drive, LPSTR dir, LPSTR name, LPSTR ext);

#define osflsz os_filelength
long os_filelength (int fh);
long os_filelength1(int fh, int blksz);

int os_chsize (int fh, long newsize, int blksz);

#define osgetcurdir os_getcurdir
 int os_getcurdir(int drive, LPSTR szDir);

#define osgetcwd os_getcwd
 LPSTR os_getcwd (LPSTR buf, int len);

# define osunlink os_unlink
  A2EXT int A2PAS os_unlink (LPSTR fname);

#if !defined(__FOR_WINDOWS__)

# define osgetdisk os_getdisk
  int os_getdisk(void);

# define ossetdisk os_setdisk
  int os_setdisk(int drive);

  int os_remote (int drive);

# define osdiskfree os_diskfree
  long os_diskfree (int drive);
  long os_diskfree1(char *pszDir, int blksize);

  int osChkSpace (LPSTR fname);

  int os_fattr (LPSTR fname, int *attrib, int mode);

# define osftime os_ftime
  int os_ftime (LPSTR fname, FTIME *ft, int mode);
  int os_ftime0(int fh, unsigned short *dt, unsigned short *tm, int set);

# define osrename os_rename
  A2EXT int A2PAS os_rename (LPSTR oldname, LPSTR newname);

# if !defined(__FOR_WINDOWS__)
#  if defined(__FOR_DOS__) || defined(__FOR_DPMI16__) || defined(NODLL)
#   define oststdr os_tstdr
    int os_tstdr (int drive);
#  endif
# endif

# define osfindfirst os_findfirst
  int os_findfirst (LPSTR fname, FFBLK *ffblk, int attrib);

# define osfindnext os_findnext
  int os_findnext  (FFBLK *ffblk);

int os_getlogicaldrives(char *buffer, size_t bsz, int mode);
enum os_getlogicaldrives_modes
{
   GLD_letters = 0*0x01,
   GLD_text    = 1*0x01,
   GLD_types   = 1*0x02
};

#endif /* !(__FOR_WINDOWS__) */

V_PTR os_malloc (unsigned size);
V_PTR os_calloc (unsigned nelem, unsigned elsize);
void os_free (V_PTR block);
long os_getmemfree(void);
int os_memlink (int on);
int os_freelink (void);
int os_checklinkmem (void);

#ifdef __FOR_DOS__
# ifdef A2SYSPROG
   extern V_PTR (*a2_ext_malloc)(unsigned size);
   extern int (*a2_ext_free)(V_PTR block);
# endif
int a2memUMB (int set);
int a2memEMS (int set);
int a2memVID (int set);
#else
# define a2memUMB(set) a2dummy_1()
# define a2memEMS(set) a2dummy_1()
# define a2memVID(set) a2dummy_1()
#endif

#define osdatime os_datime
void os_datime (struct a2datetm *pdt);
void os_setdatime(struct a2datetm *pdt);

void os_sleep ( double seconds );

#ifndef __STRIP_TV__

#define M_bioskey os_keybd
int os_keybd (int cmd);

int os_setperiodfunc (int ticks, int (*pfunc)(void));
/* if pfunc return nonzero value it placed into keyboard buffer */

#endif

#if defined(__FOR_DPMI16__) || ((defined(__FOR_DPMI32__) || defined(__FOR_WIN32__)) && defined NODLL)
#define swapexec a2emu_swapexec
int a2emu_swapexec(char *prog, char *parms);
#endif

unsigned cdecl A2Wgtimc (void);
void     cdecl A2Wbell  (void);
int      cdecl A2Wgvmode(void);

#if defined(REGMODE) /* && defined(_DEBUG) */
  extern int A2REG(int op, long lv,void *pv, long lv1,long lv2,void *pv1,void *pv2);
#else
# define A2REG(op,lv,pv,lv1,lv2,pv1,pv2)
#endif

#ifdef __cplusplus
}
#endif

#pragma pack (pop)
#endif /* H_A2SYSTEM */
