/**
  \file   a2uline.h
  \brief     'stream'  
  \author  
  \date   13.12.1990
*/

#ifndef H_A2ULINE
#define H_A2ULINE
#pragma pack (push, 8)

#include "a2sysdef.h"
/*---------- a2uline.h  flow control header file ----------*/
/*=================== Flow type flags =====================*/
#define UL_STR   0x01   /* standard stream - stdout        */
#define UL_MEM   0x02   /* memory flow                     */
#define UL_LDS   0x04   /* LDSC flow                       */
#define UL_FLB   0x08   /* beginning of the file           */
#define UL_FLE   0x10   /* append to the file              */
#define UL_ARR   0x20   /* multiple flow                   */
#define UL_PRN   0x40   /* to the printer                  */
#define UL_WND   0x80   /* to the window on the screen     */
#define UL_SCR   0x8000U/* to printer, don't use form feed */
#define UL_PAUSE 0x4000 /* wait for keystroke at
                                 the beginning of new page */
/* ===================== Constants ======================= */
#define UL_LMAX   512   /* max string length               */
#define ER_NOCB  -1000  /* ULCB not found                  */
#define ER_DOUT  -1     /* I/O error                       */
#define ER_ULCB  -2     /* ULCB is corrupted               */
/* ================== main structures ==================== */
#include "a2tvdef.h"          /* TINT, LDSC, WCB, PCB, DOC */
/* ------ Help window description for Alt and Ctrl : ----- */
typedef struct
     {unsigned char color;
      unsigned char bigcol;
      LDSC *text;
     } W_HELP;
/* --------- Description of the help function  ----------- */
typedef struct
     {int (*fun)(void *parm, unsigned key);    /* function */
      void *parm;                 /* pointer to parameters */
     } F_HELP;
#define A2_MXHLIST  5

/* ------- Common variables for the module a2uline ------- */
#ifndef  A2ULINE
#define  A2ULINE
extern unsigned *a2helpkey;    /* pointer to table help code,
                                          default: F1_F, 0 */

/* pointer to conversion table: code1i, code1o, ..., 0     */
/* codeXi is transformed to codeXo, 0 - end of table       */
extern unsigned *a2trant;

/* ------ Pointer to help windows for Alt and Ctrl ------- */
extern W_HELP *a2althlp, *a2cnthlp; /* help for Alt & Ctrl */
/* -------- Array of function for help processing -------- */
extern int a2_phlist; /* seq.number of the current function*/
extern F_HELP a2_hlist[A2_MXHLIST];

#endif

#ifndef _ULCB_T
#define _ULCB_T
/* ------------------- ULCB description ------------------ */
#define SZSTR    9      /* max length of flow name         */
#define SZLB     4      /* ULCB tag length                 */
struct a2ulcb                /* uline control block        */
     { char label[SZLB];     /* tag - "UCB"                */
       short flags;          /* flow type                  */
       char stream[SZSTR];   /* flow name (<= 8 char)      */
       struct a2ulcb *next;  /* next ULCB                  */
       struct a2ulcb *prev;  /* previous ULCB              */
                                       /* page description */
       short e__lpage;      /* page size                   */
       short e__npage;      /* current page number         */
       short e__nrow;       /* current line                */
       PCB *fe;             /* first PCB in output chain   */
       PCB *le;             /* last PCB in output chain    */
       DOC *doc;            /* processed document          */
       int (*ufc)(ULCB  *); /* user close function         */
       void *cb;            /* pointer to output ctrl block*/
       short LineInOut;               /* line in out chain */
       unsigned AnyKey ;/* code of answer for "PRESS ANY KEY" */
     };
typedef struct a2ulcb ULCB;
#endif
/*=========== Output control blocks description ===========*/
typedef struct
      {char *memory;    /* memory address                  */
       size_t lmax;     /* max string length               */
      }ULMEM;
typedef struct
      {LDSC *ldsc; /*pointer to the first LDSC in the chain*/
       char *str;       /* beginning of intermediate line  */
       size_t lmax;     /* max string length               */
       short flags;
       void *store;     /* old LDSC or window              */
      }ULLDSC;
#define UF_GM  0x01 /* memory for str is already allocated */
#define SIZE_IND 256
typedef struct
      {LDSC *ldsc; /*pointer to the first LDSC in the chain*/
       char *str;       /* beginning of intermediate line  */
       size_t lmax;     /* max string length               */
       short flags;
       void *store;     /* old LDSC or window              */
       TINT pos;    /* coordinates of window on the screen */
       TINT size;       /* size of output window           */
       char atr;        /* colors                          */
       unsigned     /* array: part type, number of strings */
            char *ind; /* number of elements =
                                       SZLINE+sizeof(LDSC) */
      }ULWND;
typedef struct
      {void *stream; /* pointer to the file control block  */
      }ULFILE;
typedef struct
      {char str1[SZSTR];
       char str2[SZSTR];
       char str3[SZSTR];
      }ULARR;


union CB
     {ULMEM um;
      ULFILE ufb;
      ULARR af;
      ULLDSC ul;
      ULWND uw;
     };

#ifdef __cplusplus
extern "C" {
#endif
/* ----------- List of colors for a2confirm -------------- */
                          /* type:default(background/text) */
extern unsigned char a2_etype; /* 'e':0x4E(red/yellow)     */
extern unsigned char a2_itype; /* 'i':0x71(cyan/blue)      */
extern unsigned char a2_qtype; /* 'q':0x1F(blue/white)     */
/* ------------ a2ulop - open of output flow ------------- */
ULCB *a2ulop(char *stream);
/* stream := flow name (less then 8 characters) or NULL    */
/* ------- a2uloprn - open output printer flow ----------- */
ULCB *a2uloprn(char *stream, size_t lmax);
/* stream := flow name (less then 8 characters) or NULL    */
/* ----------- a2ulopst - open flow to memory ------------ */
ULCB *a2ulopst(char *stream, char *str, size_t lmax);
/* stream := flow name (less then 8 characters) or NULL    */
/* str - memory for output, lmax - max length              */
/* ------------- a2ulopld - open LDSC flow --------------- */
EXTERNFUNC ULCB * EXPORTFUNC a2ulopld(char *stream,
                                 char *str, size_t lmax);
/* stream := flow name (less then 8 characters) or NULL    */
/* str :=  beginning of intermediate line                  */
/* ------------- a2ulopwn - open window flow ------------- */
ULCB *a2ulopwn(char *stream, TINT *pos, TINT *size,
                                          unsigned char atr);
/* stream := flow name (less then 8 characters) or NULL    */
/* pos,size-coordinates and size of the window or NULL,
                                                 atr-color */
/* ----------- a2ulopfb - open flow to file -------------- */
EXTERNFUNC ULCB * EXPORTFUNC a2ulopfb(char *stream,
                                            char *file);
/* stream := flow name (less then 8 characters) or NULL    */
/* file - file name                                        */
/* ---------- a2ulopfe - open flow to append file -------- */
EXTERNFUNC ULCB * EXPORTFUNC a2ulopfe(char *stream,
                                            char *file);
/* stream := flow name (less then 8 characters) or NULL    */
/* file - file name                                        */
/* ------ a2ulopm2 - open multiple flow (for 2 flow) ----- */
EXTERNFUNC ULCB * EXPORTFUNC a2ulopm2(char *stream,
                               char *st1, char *st2);
/* stream := flow name (less then 8 characters) or NULL    */
/* st1, st2 - names of flows where
                              information will be output to*/
/*- a2ulopm3 - a2ulopm2 - open multiple flow (for 3 flow) -*/
ULCB *a2ulopm3(char *stream, char *st1, char *st2, char *st3);
/* stream := flow name (less then 8 characters) or NULL    */
/* st1, st2, st3 - names of flows where
                             information will be output to */
/*------------ a2getulcb - get pointer to ULCB ------------*/
EXTERNFUNC ULCB * EXPORTFUNC a2getulcb(char *stream);
/* ----------- a2ulsetz - clear flow counter --------------*/
ULCB *a2ulsetz(char *stream);
/* a2uldfre - deallocaton of document buffers ------------ */
/* This function is used by  a2mdfree                      */
void a2uldfre(DOC *doc);
/* ----------------- a2ulcl - close flow ------------------*/
EXTERNFUNC int EXPORTFUNC  a2ulcl(char *stream);
/* -------- a2ulput - formatted output to the flow ------- */
int a2ulput(char *stream, char *format, ...);
/*- a2ulputv - sends formatted output using argument list -*/
int a2ulputv(char *stream, char *format,void *argptr);
/* ---- a2uline - formatted output to "PRINT" the flow --- */
int a2uline(char *format, ...);
/* ----- a2ulclr - free LDSC chain (created by uline) ---- */
void a2ulclr(void);
/*- a2ulgetl - get previously output messages (LDSC flow) -*/
LDSC *a2ulgetl(void);
/*------------------- a2GetCodeOfAnyKey -------------------*/
int a2GetCodeOfAnyKey(ULCB *ulcb) ;
/*------------------- a2ClearCodeOfAnyKey -----------------*/
void a2ClearCodeOfAnyKey(ULCB *ulcb) ;
/* ----------------- HELP system function ---------------- */
/* ------ a2hlchng - deactivate/delete/replace/add ------- */
#define A2H_OFF   0x11
#define A2H_DEL   0x21
#define A2H_UPDT  0x31
#define A2H_ADD   0x40
int a2hlchng(int reg, int (*f)(void *, unsigned), void **hparm);
/* reg = A2H_OFF | A2H_DEL | A2H_UPDT | A2H_ADD            */
/* --- f - this function is called if F1 key is pressed -- */
/* hparm - function parameters                             */
/* return help fuction number or -2 if an error occurred   */
/* -- a2outhlp - output help at the bottom of the screen --*/
int a2outhlp(W_HELP *w_help, void **buf);
/* w_help - window help decriptor,
              buf - buffer to save overlapped area;        */
/* if w_help==NULL window will not be displayed;           */
/*if buf==NULL or *buf!=NULL overlapped area will not saved*/
/*---------------------------------------------------------*/
/*------------ a2IntMouse - mouse interface ---------------*/
#define  a2M_ChNE  0x01           /* check new mouse event */
#define  a2M_ChCE  0x02       /* check current mouse event */
#define  a2M_GtTR  0x04                  /* get number row */
#define  a2M_GtTC  0x08               /* get number column */
#define  a2M_ClE   0x10               /* clear mouse event */
#define  a2M_ChRE  0x20        /* check region mouse event */
int a2IntMouse(int flags);
/* ------- a2key - keybord interface (like bioskey) ------ */
int a2key(int cmd);
/*--------------- a2esckey - check for ESC ----------------*/
int a2esckey(void);
/* ------------- a2ulstr - restore bottom line ----------- */
void a2ulstr(void);
/* is used after a2confirm with reg='n'                    */
/* -------------- a2confirm - confirmation --------------- */
EXTERNFUNC int EXPORTFUNC  a2confirm(char *text,
                                  char reg, char type, ...);
/* text: poiner to message or NULL, if text has been output
                      by uline function to the LDSC chaine */
/* reg: 'n' - do not wait for keystroke (see a2ulstr),     */
/*      'a' or 'm'- wait for any keystroke                 */
/*             if 'm' is specified -1 will be returned     */
/*                   at the attempt to use empty LDSC chain*/
/*             if 'a' is error message "empty line" will be*/
/*                    output if at the attempt to use empty*/
/*                    LDSC chain                           */
/*      'c' - wait for one of the following keystrokes:    */
/*                   "Y","N" or ESC                        */
/* type determines color                                   */
/*      'i' - information,                                 */
/*      'q' - question,                                    */
/*      'e' - error message                                */
/* before output overlapped text is saved                  */
/*                               and before it is restored */
/* Return:                                                 */
/*    if reg= 'a' or 'm'                                   */
/*          Extended Scan Code or -1 (empty LDSC chain)    */
/*    if reg= 'c'                                          */
/*          0 - YES,                                       */
/*         -1 - NO,                                        */
/*         -2 - ESC                                        */
EXTERNFUNC int EXPORTFUNC a2confirmTr(char *text,
                                  char reg, char type, ...);

/* ------------- a2opmsg -  message file open ------------ */
void *a2opmsg(char *name);
/* name - file name (without extension)                    */
/* Return pointer to FILE                                  */
/* ------------- a2clmsg - message file close ------------ */
int a2clmsg(void *fl);
/* -------------- a2prmsg - message output --------------- */
int a2prmsg(char *stream, char type, char *name, int num, ...);
/* stream - output flow name                               */
/* type - message source:                                  */
/*    'f' - name is file name                              */
/*    'm' - name is pointer to message buffer(see a2rdmsg) */
/* num - message number                                    */
/*-------- a2troubl - output trouble messages  ------------*/
int a2troubl(int regime, char *text);
/*---------------------------------------------------------*/
/* -a2rdmsg - read a group of message from file to buffer -*/
void *a2rdmsg(int num1, int num2, char *file);
/* num1, num2 - from, to numbers;                          */
/* file - file name                                        */
/* Return pointer to message buffer                        */
/* --------- a2frmsg - deallocate message buffer --------- */
void *a2frmsg(char *buft);
/* -------------- a2getmsg - get message ------------------*/
char *a2getmsg(char t, char *sname, int num,
                                      char *ubuf, int maxln);
/* t - message source:                                     */
/*    'f' - name is file name                              */
/*    'm' - name is pointer to message buffer(see a2rdmsg) */
/* sname - file name without extension if type='f'         */
/*         pointer to message buf if type='m' (see a2rdmsg)*/
/* num - message number                                    */
/* ubuf - buffer for message                               */
/* maxln - buffer length                                   */
/* ---------- a2ldmsg - read message into LDSC ----------- */
LDSC *a2ldmsg(char tmsg, char *name, int num, LDSC *ldsc);
/* tmsg,name,num - have the same meaning as in a2msgpr;    */
/* ldsc - pointer to the first LDSC in the chain or NULL   */
/* ----- a2conmsg - analog of a2confirm for messages ----- */
int a2conmsg(char tmsg, char *name, int num,
                                   char reg, char type, ...);
/* tmsg,name,num,reg,type - have the same meaning as
                                            in a2msgpr     */
int a2conmsgTr(char tmsg, char *name, int num,
                                   char reg, char type, ...);
/*-------- a2langmsg - language of messages ---------------*/
void a2langmsg(char *answer);
/*-------- a2GetRegNumber - number of registration --------*/
long a2GetRegNumber(void);
/*-------- a2GetUlTb -  return adress uline table ---------*/
void *a2GetUlTb(int type) ;
#ifdef __cplusplus
}
#endif
/*---------------------------------------------------------*/
#pragma pack (pop)
#endif /* H_A2ULINE */
